;******************************************************************************
; File Name:	 ch8_e1.asm
; Target System: C240x Evaluation Board
; Description: 	This program uses the SCI module to implement a simple
; 				asynchronous communications routine. The SCI is initialized
;				to operate in idleline mode with 8 character bits, 1 stop
;				bit, and odd parity. The SCI Baud Rate Registers (BRR) are
;				set to transmit and receive data at 19200 baud. The SCI
;				generates an interrupt every time a character is loaded into
;				the receive buffer (SCIRXBUF). The interrupt service routine
;				(ISR) reads the receive buffer and determines if the carriage
; 				return button, <CR>,has been pressed. If so, the character
;				string is compared with the value of password expected. If the
;				password is correct, the character string "Right" is transmitted
;				otherwise, the character string "Wrong" is transmitted.
;
; Date:			01/13/2004
; Author:		Min Dai
;******************************************************************************

;******************************************************************************
;					  SYSTEM OPTIONS
;******************************************************************************
real_time	.set	0      		; 1 for real time mode, otherwise set 0
LENGTH  	.set 	16 			; length of the data stream to be transmitted
LENGTH2 	.set 	8
PASWD   	.set 	4321h  		; 4 digit numeric password
B2_SADDR	.set	60h			; On-chip DRAM B2 starting address (data memory)

;******************************************************************************

;-----------------------------------------------------------------------------
; External references
;-----------------------------------------------------------------------------
		.include 	"f2407.h"
;		.global 	MON_RT_CNFG

		.ref		SYS_INIT

;-----------------------------------------------------------------------------
; Local Variable Declarations
;-----------------------------------------------------------------------------
		.def		GPR0				;General purpose register.

		.bss		GPR0,1				;General purpose register.
  		.bss 		RX_PASWD,1			;Storage for received password
		.bss 		CHR_CNT,1			;Counter for number of characters received

;==============================================================================
; V E C T O R    T A B L E    ( including RT monitor traps )
;==============================================================================
;        .include 	"c200mnrt.i" 		; Include conditional assembly options.

		.global		_c_int0,PHANTOM,GISR1,GISR2,GISR3,GISR4,GISR5,GISR6

;==============================================================================
; M A I N   C O D E  - starts here
;==============================================================================
		.text
_c_int0:
		CALL		SYS_INIT                ;DSP initialization

;----------------------------------------------------------
; Initialise the Real time monitor
;----------------------------------------------------------

		POINT_PG0

;---Real Time option---------------
	.if (real_time)
		CALL		MON_RT_CNFG				;For Real-Time
	.endif
;----------------------------------

;----------------------------------------------------------
; System Interrupt Init.
;----------------------------------------------------------

;---Real Time option --------------------------------------------------
	.if (real_time)
		SPLK	#0000000001000001b,IMR	;En Int lvl 1 & 7 for SCI and RT
		        ;|||||||||!||||||
			    ;5432109876543210
	.endif


	.if (real_time != 1)
		SPLK	#0000000000000001b,IMR	;Enable Int 1 only for SCI
			    ;||||!!!!||||!!!!
			    ;5432109876543210
	.endif

		SPLK	#0FFFFh, IFR			;Clear any pending Ints
;-----------------------------------------------------------------------

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Setup shared I/O pins
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        LDP     #DP_PF2                 ;set data page

        SPLK    #0000000000000011b,MCRA ;set SCI pins
*                ||||||||||||||||
*                FEDCBA9876543210
* bit 15        0:      0=IOPB7,     1=TCLKINA
* bit 14        0:      0=IOPB6,     1=TDIRA
* bit 13        0:      0=IOPB5,     1=T2PWM/T2CMP
* bit 12        0:      0=IOPB4,     1=T1PWM/T1CMP
* bit 11        0:      0=IOPB3,     1=PWM6
* bit 10        0:      0=IOPB2,     1=PWM5
* bit 9         0:      0=IOPB1,     1=PWM4
* bit 8         0:      0=IOPB0,     1=PWM3
* bit 7         0:      0=IOPA7,     1=PWM2
* bit 6         0:      0=IOPA6,     1=PWM1
* bit 5         0:      0=IOPA5,     1=CAP3
* bit 4         0:      0=IOPA4,     1=CAP2/QEP2
* bit 3         0:      0=IOPA3,     1=CAP1/QEP1
* bit 2         0:      0=IOPA2,     1=XINT1
* bit 1         1:      0=IOPA1,     1=SCIRXD
* bit 0         1:      0=IOPA0,     1=SCITXD

;=====================================================================
; Initialize user variables
;=====================================================================
		LDP		#RX_PASWD
		SPLK 	#0, RX_PASWD
		SPLK 	#0, CHR_CNT
;=====================================================================
; Initialize B2 RAM to zeros.
;=====================================================================
		LAR 	AR2,#B2_SADDR		; AR2 <- B2 start address
		MAR 	*,AR2 				; Set ARP=AR2
		LACL 	#0 					; Set ACC = 0
		RPT 	#1Fh 				; Set repeat cntr for 31+1 loops
		SACL 	*+ 					; Write zeros to B2 RAM
;=====================================================================
; Initialize DATAOUT with data to be transmitted.
;=====================================================================
		LAR 	AR2,#B2_SADDR 		; Reset AR2 B2 start address
		RPT 	#(LENGTH - 1) 		; set repeat counter for LENGTH loops
		BLPD 	#TXDATA,*+	 		; loads B2 with TXDATA

;=====================================================================
; INITIALIZATION OF INTERRUPT DRIVEN SCI ROUTINE
;=====================================================================
SCI_INIT:
		LDP 	#DP_PF1
		SPLK 	#0027h, SCICCR 		;1 stop bit,odd parity,8 char bits,
									;async mode, idleline protocol
		SPLK 	#0003h, SCICTL1 	;Enable TX, RX, internal SCICLK,
									;Disable RX ERR, SLEEP, TXWAKE
		SPLK 	#0002h, SCICTL2 	;Enable RX INT,disable TX INT
		SPLK 	#0000h, SCIHBAUD
		SPLK 	#194, SCILBAUD 		;Baud Rate=19200 b/s (30 MHz CLKOUT)
									; 30000000/19200/8-1 = 194
		SPLK 	#0000h, SCIPRI 		;Set TXD & RXD INT high priority
		SPLK 	#0023h, SCICTL1 	;Enable TX, RX, internal SCICLK,
									;Disable RX ERR, SLEEP, TXWAKE
									;Reset SCI
		LAR 	AR0, #SCITXBUF 		;Load AR0 with SCI_TX_BUF address
		LAR 	AR1, #SCIRXBUF 		;Load AR1 with SCI_RX_BUF address
		LAR 	AR2, #B2_SADDR 		;Load AR2 with TX data start address

		CLRC 	INTM 				;Enable interrupts

WAIT:	B    	WAIT

;=====================================================================
; I S R  		INT1 ISR
; Description: 	The GISR1 first determines if the SCI RXINT caused
; 				the interrupt. If so, the SCI Specific ISR reads the
;				character in the RX buffer. If the character received
;				corresponds to a carriage return, <CR>, the character
;				string received is compared with the expected value of
;				password. If the value matches, "Right" is transmitted,
;				else "Wrong" is transmitted. If the character received
;				does NOT correspond to a carriage return, <CR>, then
;				the ISR returns to the main program without transmitting
;				a character string. If the SCI RXINT did not cause an
;				interrupt, then the value 0x0badis stored in the
; 				accumulator and program gets caught in the BAD_INT
;				endless loop.
;=====================================================================

GISR1	LDP 	#DP_PF1
		LACL 	PIVR 			;Load peripheral INT vector address
		SUB 	#0006h 			;Subtract RXINT offset from above
		BCND 	SCI_ISR,EQ 		;verify RXINT initiated interrupt
		B 		BAD_INT 		;Else, bad interrupt occurred
SCI_ISR LDP  	#CHR_CNT
		LARP	1 				;Set ARP=AR1
		LACC 	*	 			;Load ACC w/RX buffer character
		SUB 	#000Dh 			;Check if <CR> has been pressed:
		BCND 	XMIT_CHAR,EQ 	;YES? Transmit data.
		LACC 	*
		SUB  	#30h			;Extract the actual value from the
   								;ASCII equivalent of the number.
		RPT  	CHR_CNT
		SFL
		SFR
		OR   	RX_PASWD
		SACL 	RX_PASWD
		LACC 	CHR_CNT
		ADD  	#4
		SACL 	CHR_CNT
		B 		ISR_END 		;N0? Return from INT1_ISR.

XMIT_CHAR
		SPLK 	#0, CHR_CNT
		LAR 	AR2, #B2_SADDR
		LACC 	RX_PASWD
		SUB  	#PASWD
		SPLK 	#0, RX_PASWD	;Check if received no. = password
		BCND 	XMIT_CHAR1, EQ
		LAR  	AR2, #(B2_SADDR + LENGTH2) 	; if not point to starting
  											; address of the string "Wrong"

XMIT_CHAR1
		LARP	2
		LACC 	*+,AR0 			;Load char to be xmitted into ACC
		BCND 	ISR_END1,EQ 		;Check for Null Character
								;YES? Return from GISR1.
		SACL 	*,AR2 			;NO? Load char into xmit buffer.
XMIT_RDY
		LDP  	#DP_PF1
		BIT 	SCICTL2,BIT7 	;Test TXRDY bit
		BCND 	XMIT_RDY,NTC 	;If TXRDY=0,then repeat loop
		B 		XMIT_CHAR1 		;else xmit next character
ISR_END1
		B		ISR_END
ISR_END LAR 	AR2, #B2_SADDR 	;Reload AR2 w/ TX data start address

		CLRC 	INTM 			;Clear INT Mask flag
		RET 					;Return from GISR1

BAD_INT LACC 	#0BADh 			;Load ACC with "bad"
		B 		BAD_INT 		;Repeat loop

;==============================================================================
; I S R  -  PHANTOM
;
; Description:	Dummy ISR, used to trap spurious interrupts.
;
; Modifies:
;
; Last Update:	16-06-95
;==============================================================================
PHANTOM B	PHANTOM
;GISR1 	RET
GISR2 	RET
GISR3 	RET
GISR4 	RET
GISR5 	RET
GISR6	RET

; Initialize Transmit Data for Interrupt Service Routine
  		.data
TXDATA  .word 0052h 	;Hex equivalent of ASCII character R
		.word 0069h 	;Hex equivalent of ASCII character i
		.word 0067h 	;Hex equivalent of ASCII character g
  		.word 0068h 	;Hex equivalent of ASCII character h
		.word 0074h 	;Hex equivalent of ASCII character t
		.word 000Ah		;Hex equivalent of line feed
		.word 000Dh		;Hex equivalent of carriage return
 		.word 0000h		;Hex equivalent of NULL

WRONG	.word 0057h 	;Hex equivalent of ASCII character W
		.word 0072h 	;Hex equivalent of ASCII character r
		.word 006fh 	;Hex equivalent of ASCII character o
		.word 006eh 	;Hex equivalent of ASCII character n
		.word 0067h 	;Hex equivalent of ASCII character g
		.word 000Ah		;Hex equivalent of line feed
		.word 000Dh		;Hex equivalent of carriage return
		.word 0000h		;Hex equivalent of NULL
